;
; FME-7 big RAM test for NES
; Copyright 2015 Damian Yerrick
;
; Copying and distribution of this file, with or without
; modification, are permitted in any medium without royalty provided
; the copyright notice and this notice are preserved in all source
; code copies.  This file is offered as-is, without any warranty.
;

.include "nes.inc"
.include "global.inc"

OAM = $0200
MAX_WRAM_BANKS = 16

.segment "ZEROPAGE"
nmis:           .res 1
cur_keys:       .res 2
new_keys:       .res 2

; Measurements
open_bus_ok:    .res 1

string_buffer:  .res 16

.segment "CODE"
;;
; This NMI handler is good enough for a simple "has NMI occurred?"
; vblank-detect loop.
.proc nmi_handler
  inc nmis
  rti
.endproc
.proc irq_handler
  rti
.endproc

.proc main
  jsr load_main_palette
  jsr run_tests_once

disp_results:
  lda #>results_msg
  ldy #<results_msg
  jsr cls_puts_multiline
  jsr display_results
  
  jsr press_A

  lda #>page1_msg
  ldy #<page1_msg
  jsr cls_puts_multiline
  jsr press_A
  lda #>page2_msg
  ldy #<page2_msg
  jsr cls_puts_multiline
  jsr press_A
  lda #>page3_msg
  ldy #<page3_msg
  jsr cls_puts_multiline
  jsr press_A
  jmp disp_results
.endproc

.proc present
  ; Wait for a vertical blank
  lda nmis
vw3:
  cmp nmis
  beq vw3
  lda #0
  sta OAMADDR
  lda #>OAM
  sta OAM_DMA
  
  ; Turn the screen on
  ldx #0
  ldy #0
  lda #VBLANK_NMI|BG_0000|OBJ_0000
  sec
  jmp ppu_screen_on
.endproc

.proc press_A
  jsr present
  ; Does player want to quit?
  jsr read_pads
  lda new_keys
  and #KEY_A|KEY_START
  beq press_A
  rts
.endproc

.proc load_main_palette
  ; seek to the start of palette memory ($3F00-$3F1F)
  ldx #$3F
  stx PPUADDR
  ldx #$00
  stx PPUADDR
copypalloop:
  lda initial_palette,x
  sta PPUDATA
  inx
  cpx #initial_palette_end-initial_palette
  bcc copypalloop
  rts
.endproc

.proc display_results
  lda #>open_bus_ok
  sta $05
  lda #<open_bus_ok
  sta $04
  lda #$21
  ldx #$27
  ldy #$01
  jsr hexdump8
  lda #>ppu_readback_values
  sta $05
  lda #<ppu_readback_values
  sta $04
  lda #$21
  ldx #$A7
  ldy #$08
  jsr hexdump8
  lda #>open_bus_values
  sta $05
  lda #<open_bus_values
  sta $04
  lda #$22
  ldx #$27
  ldy #$08
  jsr hexdump8
  lda #>minbits_40
  sta $05
  lda #<minbits_40
  sta $04
  lda #$22
  ldx #$A7
  ldy #$08
  jsr hexdump8
  rts
.endproc

.segment "RODATA"
initial_palette:
  .byt $17,$27,$20,$37
initial_palette_end:

results_msg:
  .byte "NES open bus test",10
  .byte "Copr. 2015 Damian Yerrick",10
  .byte "",10
  .byte "code:",10
  .byte "PPU readback result",10
  .byte "",10
  .byte "Open bus result",10
  .byte "",10
  .byte "Controller bits",10
  .byte "",10
  .byte "",10
  .byte " Press Start for explanation",0

page1_msg:
  .byte "I'm trying to make a ROM",10
  .byte "that distinguishes the NES",10
  .byte "from the Famicom and lists",10
  .byte "which controllers are",10
  .byte "plugged into the system.",10
  .byte "I planned to do this using",10
  .byte "distinct open bus behavior,",10
  .byte "but the PowerPak and several",10
  .byte "emulators are disagreeing",10
  .byte "again.",10
  .byte "",10
  .byte "[1/3]",0

page2_msg:
  .byte "Open bus occurs when nothing",10
  .byte "puts a 0 or 1 on a bit of",10
  .byte "the data bus during a read.",10
  .byte "Capacitance holds the old",10
  .byte "voltages in place for the",10
  .byte "CPU to use as the value.",10
  .byte "Reading nonexistent memory,",10
  .byte "for example, usually leaves",10
  .byte "the instruction's last byte",10
  .byte "on the bus, which on 6502 is",10
  .byte "the address's high byte.",10
  .byte "[2/3]",0

page3_msg:
  .byte "But the PowerPak has been",10
  .byte "known to interfere with",10
  .byte "open bus before.  Pull-up",10
  .byte "resistors for 3.3 V to 5 V",10
  .byte "conversion tend to remove",10
  .byte "old bits from the data bus.",10
  .byte "Mindscape games would fail",10
  .byte "to see presses because they",10
  .byte "expected open bus on unused",10
  .byte "bits of the controller port.",10
  .byte "",10
  .byte "[3/3]Press Start for results",0

tv_system_names_lo: .lobytes ntsc_name, pal_name
tv_system_names_hi: .hibytes ntsc_name, pal_name
ntsc_name: .byte "NTSC (60 Hz)",0
pal_name:  .byte "PAL (50 Hz) ",0

NTSC_HSYNC_CENTI = 1574580  ; NTSC hsync in cHz
NTSC_HSYNC_4096  = 64494    ; 4096 * NTSC hsync in kHz
PAL_HSYNC_CENTI  = 1560218  ; PAL hsync in cHz
PAL_HSYNC_4096   = 63907    ; 4096 * PAL hsync in kHz

hsync_centi_lo:   .lobytes   NTSC_HSYNC_CENTI, PAL_HSYNC_CENTI
hsync_centi_hi:   .hibytes   NTSC_HSYNC_CENTI, PAL_HSYNC_CENTI
hsync_centi_bank: .bankbytes NTSC_HSYNC_CENTI, PAL_HSYNC_CENTI
hsync_4096_lo: .hibytes NTSC_HSYNC_4096, PAL_HSYNC_4096
hsync_4096_hi: .hibytes NTSC_HSYNC_4096, PAL_HSYNC_4096

; Include the CHR ROM data
.segment "CHR"
  .res $0400                           ; 00-3F: empty
  .incbin "obj/nes/fizztertiny16.chr"  ; 40-FF: font
  .res $1000                           ; Right pattern table: empty

; Input tests ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.segment "CODE"

.proc run_tests_once
  jsr ensure_open_bus_ok
  sta open_bus_ok
  jsr detect_controller_wires

  rts
.endproc


