.include "nes.inc"
.include "global.inc"

.segment "BSS"
ppu_readback_values: .res 8
open_bus_values:     .res 8

.segment "ZEROPAGE"
; Minimum of all bit values from $4016/$4017 over 32 reads with $40 open bus
minbits_40: .res 2
; Minimum of all bit values from $4016/$4017 over 32 reads with $3F open bus
minbits_3F: .res 2
; Maximum of all bit values from $4016/$4017 over 32 reads with $40 open bus
maxbits_40: .res 2
; Maximum of all bit values from $4016/$4017 over 32 reads with $3F open bus
maxbits_3F: .res 2

.segment "CODE"

.proc ensure_open_bus_ok
  ; Write 20 00 FF to the first nametable, so that reads
  ; from $2007 return xx 20 00 FF
  ldx #$20
  lda #$00
  sta PPUCTRL  ; disable NMI; set VRAM increment to +1 (not +32)
  sta PPUMASK  ; disable rendering
  bit PPUSTATUS
  ldy #$FF
  stx PPUADDR
  sta PPUADDR
  sta PPUDATA  ; $2000 = $00
  sty PPUDATA  ; $2001 = $FF

  ; Basic readback through $2007
  stx PPUADDR
  sta PPUADDR
  bit PPUDATA  ; priming read
  ldy PPUDATA
  sty ppu_readback_values+0
  ldy PPUDATA
  sty ppu_readback_values+1
  
  ; And through mirrors of $2007
  stx PPUADDR
  sta PPUADDR
  bit PPUDATA  ; priming read
  ldy $3F07
  sty ppu_readback_values+2
  ldy $3F07
  sty ppu_readback_values+3
  stx PPUADDR
  sta PPUADDR
  bit PPUDATA  ; priming read
  ldy $3F17
  sty ppu_readback_values+4
  ldy $3F17
  sty ppu_readback_values+5
  
  ; And through open bus primed by a mirror of $2007
  stx PPUADDR
  sta PPUADDR
  bit PPUDATA  ; priming read
  ldy $4007-$20,x
  sty ppu_readback_values+6
  ldy $4007-$20,x
  sty ppu_readback_values+7
  cpx #$20
  beq assert20ok
  lda #$3F
  sta PPUADDR
  lda #$00
  sta PPUADDR
  ldx #$1C
  stx PPUDATA
assert20ok:

  lda $4006
  sta open_bus_values+0
  lda $4007
  sta open_bus_values+1
  lda $2000
  sta open_bus_values+2
  lda $2001
  sta open_bus_values+3
  lda $2006
  sta open_bus_values+4
  lda $3F06
  sta open_bus_values+5
  lda $3F16
  sta open_bus_values+6
  lda $4006-$20,x
  sta open_bus_values+7

  ; Previous test
  ldx #$20
  lda #$00
  ldy #$FF
  stx PPUADDR
  sta PPUADDR
  bit PPUDATA  ; priming read
  cmp PPUDATA
  bne failed1
  cpy PPUDATA
  beq test2
failed1:
  lda #1
  rts

  ; Test 2: Try to read it back through $3F17 and $3F07,
  ; which in an authentic NES are mirrors of PPU port $2007
test2:
  stx PPUADDR
  sta PPUADDR
  bit $3F17
  cmp $3F17
  bne failed2
  cpy $3F17
  bne failed2
  stx PPUADDR
  sta PPUADDR
  bit $3F07
  cmp $3F07
  bne failed2
  cpy $3F07
  beq test3
failed2:
  lda #2
  rts

  ; Read open bus from APU ports $4006 and $4007
test3:
  lda #$40
  cmp $4006
  beq test4
  lda #3
  rts

  ; Read open bus from PPU ports $2006, $3F06, and $3F16
test4:
  cpx PPUADDR
  bne fail4
  lda #$3F
  cmp $3F06
  bne fail4
  cmp $3F16
  beq test5
fail4:
  lda #4
  rts

  ; Read open bus from PPU $3F06 and APU port $4006
  ; x is still $20
test5:
  cmp $4006 - $20,x
  beq test6
  lda #5
  rts

  ; Read data from PPU and have it carry over across
  ; APU open bus.
test6:
  stx PPUADDR
  lda #$00
  sta PPUADDR
  bit PPUDATA
  cmp $4007 - $20,x
  bne fail6
  tya
  cmp $4007 - $20,x
  beq passed
fail6:
  lda #6
  rts
passed:
  lda #0
  rts 
.endproc

.proc detect_controller_wires

  ; Fill $23C0-$23FF with $3F and point the PPU there
  ldx #$20
  lda #$40
  ldy #$3F
  jsr ppu_clear_nt
  lda #$23
  sta PPUADDR
  lda #$C0
  sta PPUADDR
  bit PPUSTATUS

  lda #$FF
  sta minbits_40+0
  sta minbits_40+1
  sta minbits_3F+0
  sta minbits_3F+1
  lda #$00
  sta maxbits_40+0
  sta maxbits_40+1
  sta maxbits_3F+0
  sta maxbits_3F+1

  ; Read directly from $401x
  lda #1
  sta $4016
  ldy #32  ; allow up to 32 bits of serial
  lsr a
  sta $4016
loop401x:
  lda $4016
  pha
  and minbits_40+0
  sta minbits_40+0
  pla
  ora maxbits_40+0
  sta maxbits_40+0
  lda $4017
  pha
  and minbits_40+1
  sta minbits_40+1
  pla
  ora maxbits_40+1
  sta maxbits_40+1
  dey
  bne loop401x

  ; Read from $3F1x followed by $401x
  lda #1
  sta $4016
  ldy #32   ; allow up to 32 bits of serial
  ldx #$20  ; index to get page crossing
  lsr a
  sta $4016
loop3F1x:
  lda $4016 - $20,x
  pha
  and minbits_3F+0
  sta minbits_3F+0
  pla
  ora maxbits_3F+0
  sta maxbits_3F+0
  lda $4016 - $20,x
  pha
  and minbits_3F+1
  sta minbits_3F+1
  pla
  ora maxbits_3F+1
  sta maxbits_3F+1
  dey
  bne loop3F1x

  rts
.endproc
